package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.entity.DeptVital;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.DeptVitalService;
import com.diagbot.service.impl.DeptVitalServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.DeptVitalVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/11/22 11:45
 */
@Component
public class DeptVitalFacade extends DeptVitalServiceImpl implements DeptVitalService {

    @Autowired
    private QuestionInfoFacade questionInfoFacade;
    @Autowired
    private DeptVitalServiceImpl deptVitalServiceImpl;

    /**
     * 保存查体模板
     *
     * @param deptVitalVO
     * @return
     */
    public Boolean saveDeptVitals(DeptVitalVO deptVitalVO) {
        //先删除该科室原有模板
        UpdateWrapper<DeptVital> deptVitalUpdateWrapper = new UpdateWrapper<>();
        deptVitalUpdateWrapper.eq("dept_id", deptVitalVO.getDeptId()).
                eq("is_deleted",IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        this.update(new DeptVital(),deptVitalUpdateWrapper);

        //查找标签是否存在
        QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
        questionInfoQueryWrapper.in("id", deptVitalVO.getVitalIds()).
                eq("type", QuestionTypeEnum.Vital.getKey()).
                eq("is_deleted", IsDeleteEnum.N.getKey());
        List<QuestionInfo> questionInfoList = questionInfoFacade.list(questionInfoQueryWrapper);

        Map<Long, QuestionInfo> questionInfoMap = EntityUtil.makeEntityMap(questionInfoList, "id");
        for (Long vital : deptVitalVO.getVitalIds()) {
            if (questionInfoMap.get(vital) == null) {
                throw new CommonException(CommonErrorCode.NOT_EXISTS, "id=" + vital + "的查体标签不存在");
            }
        }

        //插入新模板
        List<DeptVital> deptVitalList = Lists.newArrayList();
        Date now = DateUtil.now();
        String userId = UserUtils.getCurrentPrincipleID();
        for (Long vital : deptVitalVO.getVitalIds()) {
            DeptVital deptVital = new DeptVital();
            deptVital.setDeptId(deptVitalVO.getDeptId());
            deptVital.setVitalId(vital);
            deptVital.setCreator(userId);
            deptVital.setGmtCreate(now);
            deptVital.setModifier(userId);
            deptVital.setGmtModified(now);
            deptVitalList.add(deptVital);
        }

        deptVitalServiceImpl.saveBatch(deptVitalList);
        return true;
    }

    /**
     * 删除科室查体模板标签
     *
     * @param deptVitalVO
     * @return
     */
    public Boolean delDeptVitalList(DeptVitalVO deptVitalVO) {
        UpdateWrapper<DeptVital> deptVitalUpdateWrapper = new UpdateWrapper<>();
        deptVitalUpdateWrapper.eq("dept_id", deptVitalVO.getDeptId()).
                in("vital_id", deptVitalVO.getVitalIds()).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        this.update(new DeptVital(), deptVitalUpdateWrapper);
        return true;
    }

    /**
     * 删除科室查体模板标签-全删除
     * @param deptId
     * @return
     */
    public Boolean delAllDeptVitalList(Long deptId) {
        UpdateWrapper<DeptVital> deptVitalUpdateWrapper = new UpdateWrapper<>();
        deptVitalUpdateWrapper.eq("dept_id", deptId).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        this.update(new DeptVital(), deptVitalUpdateWrapper);
        return true;
    }

    /**
     * 根据部门获取查体模板
     * @param deptId
     * @return
     */
    public List<QuestionInfo> getVitalsByDeptId(Long deptId) {
        QueryWrapper<DeptVital> deptVitalQueryWrapper = new QueryWrapper<>();
        deptVitalQueryWrapper.eq("dept_id", deptId).
                eq("is_deleted", IsDeleteEnum.N.getKey());
        List<DeptVital> deptVitalList = this.list(deptVitalQueryWrapper);
        List<Long> vitalIds = deptVitalList.stream()
                .map(deptVitals -> deptVitals.getVitalId())
                .collect(Collectors.toList());
        List<QuestionInfo> vitalList = Lists.newArrayList(questionInfoFacade.listByIds(vitalIds));
        return vitalList;
    }
}
